@@ -593,16 +593,17 @@ struct Grid : GridBase<T> {
   template <bool UsePbc>
   void check_size_for_points_in_box(int& du, int& dv, int& dw,
                                     bool fail_on_too_large_radius) const {
-    if (fail_on_too_large_radius) {
-      if (2 * du >= nu || 2 * dv >= nv || 2 * dw >= nw)
-        fail("grid operation failed: radius bigger than half the unit cell?");
-    }
-    if (UsePbc && !fail_on_too_large_radius) {
-      // If we'd use the minimum image convention the max would be (nu-1)/2.
-      // The limits set here are necessary for index_n() that is used below.
-      du = std::min(du, nu - 1);
-      dv = std::min(dv, nv - 1);
-      dw = std::min(dw, nw - 1);
+    if (UsePbc) {
+      if (fail_on_too_large_radius) {
+        if (2 * du >= nu || 2 * dv >= nv || 2 * dw >= nw)
+          fail("grid operation failed: radius bigger than half the unit cell?");
+      } else {
+        // If we'd use the minimum image convention the max would be (nu-1)/2.
+        // The limits set here are necessary for index_n() that is used below.
+        du = std::min(du, nu - 1);
+        dv = std::min(dv, nv - 1);
+        dw = std::min(dw, nw - 1);
+      }
     }
   }
 
@@ -669,9 +670,9 @@ struct Grid : GridBase<T> {
   void set_points_around(const Position& ctr, double radius, T value, bool use_pbc=true) {
     Fractional fctr = unit_cell.fractionalize(ctr);
     if (use_pbc)
-      use_points_around<true>(fctr, radius, [&](T& ref, double) { ref = value; });
+      use_points_around<true>(fctr, radius, [&](T& ref, double) { ref = value; }, false);
     else
-      use_points_around<false>(fctr, radius, [&](T& ref, double) { ref = value; });
+      use_points_around<false>(fctr, radius, [&](T& ref, double) { ref = value; }, false);
   }
 
   void change_values(T old_value, T new_value) {
