@@ -95,6 +95,7 @@ struct PdbReadOptions {
   bool split_chain_on_ter = false;
   bool skip_remarks = false;
 };
+// end of PdbReadOptions for mol.rst
 
 // remove empty residues from chain, empty chains from model, etc
 template<class T> void remove_empty_children(T& obj) {
@@ -866,6 +867,7 @@ struct Structure {
   std::vector<Helix> helices;
   std::vector<Sheet> sheets;
   std::vector<Assembly> assemblies;
+  std::map<int, std::vector<int>> conect_map;
   Metadata meta;
 
   CoorFormat input_format = CoorFormat::Unknown;
@@ -974,6 +976,16 @@ struct Structure {
     return std::any_of(ncs.begin(), ncs.end(), [](const NcsOp& o) { return !o.given; });
   }
 
+  void add_conect_one_way(int serial1, int serial2, int order) {
+    auto& vec = conect_map[serial1];
+    for (int i = 0; i < order; ++i)
+      vec.insert(std::upper_bound(vec.begin(), vec.end(), serial2), serial2);
+  }
+  void add_conect(int serial1, int serial2, int order) {
+    add_conect_one_way(serial1, serial2, order);
+    add_conect_one_way(serial2, serial1, order);
+  }
+
   void merge_chain_parts(int min_sep=0) {
     for (Model& model : models)
       model.merge_chain_parts(min_sep);
