@@ -99,6 +99,16 @@ struct GEMMI_DLL Mtz {
     bool is_integer() const {
       return type == 'H' || type == 'B' || type == 'Y' || type == 'I';
     }
+
+    const Column* get_next_column_if_type(char next_type) const {
+      if (idx + 1 < parent->columns.size()) {
+        const Column& next_col = parent->columns[idx + 1];
+        if (next_col.dataset_id == dataset_id && next_col.type == next_type)
+          return &next_col;
+      }
+      return nullptr;
+    }
+
     using iterator = StrideIter<float>;
     iterator begin() {
       assert(parent);
@@ -276,6 +286,12 @@ struct GEMMI_DLL Mtz {
     return UnitCell(avg[0], avg[1], avg[2], avg[3], avg[4], avg[5]);
   }
 
+  void set_spacegroup(const SpaceGroup* new_sg) {
+    spacegroup = new_sg;
+    spacegroup_number = new_sg ? spacegroup->ccp4 : 0;
+    spacegroup_name = new_sg ? spacegroup->hm : "";
+  }
+
   Dataset& last_dataset() {
     if (datasets.empty())
       fail("MTZ dataset not found (missing DATASET header line?).");
@@ -779,6 +795,9 @@ struct GEMMI_DLL Mtz {
     }
   }
 
+  /// the same as read_input(MaybeGzipped(path), with_data)
+  void read_file_gz(const std::string& path, bool with_data=true);
+
   std::vector<int> sorted_row_indices(int use_first=3) const {
     if (!has_data())
       fail("No data.");
@@ -821,12 +840,19 @@ struct GEMMI_DLL Mtz {
       data[offset + i] = static_cast<float>(hkl[i]);
   }
 
+  /// Returns offset of the first hkl or (size_t)-1. Can be slow.
+  size_t find_offset_of_hkl(const Miller& hkl, size_t start=0) const;
+
   /// (for merged MTZ only) change HKL to ASU equivalent, adjust phases, etc
   void ensure_asu(bool tnt_asu=false);
 
   /// reindex data, usually followed by ensure_asu()
   void reindex(const Op& op, std::ostream* out);
 
+  /// Change symmetry to P1 and expand reflections. Does not sort.
+  /// Similar to command EXPAND in SFTOOLS.
+  void expand_to_p1();
+
   // (for unmerged MTZ only) change HKL according to M/ISYM
   bool switch_to_original_hkl() {
     if (indices_switched_to_original)
@@ -1088,7 +1114,6 @@ Mtz read_mtz(Input&& input, bool with_data) {
   return mtz;
 }
 
-
 // Abstraction of data source, cf. ReflnDataProxy.
 struct MtzDataProxy {
   const Mtz& mtz_;
