@@ -33,6 +33,10 @@ void add_elem(py::module& m) {
              })(r2);
     }, py::arg("r2"), py::arg("B"))
     ;
+  m.def("IT92_normalize", &IT92::normalize);
+  // can't define property for py::module_, and we don't expose IT92 as class
+  m.def("IT92_get_ignore_charge", []() { return IT92::ignore_charge; });
+  m.def("IT92_set_ignore_charge", [](bool v) { IT92::ignore_charge = v; });
 
   // c4322.hpp
   using C4322 = gemmi::C4322<double>;
@@ -60,7 +64,7 @@ void add_elem(py::module& m) {
          py::arg("r2"), py::arg("B"))
     ;
 
-  // elem.hpp
+  // elem.hpp (w/ properties from it92.hpp, ...)
   py::class_<Element>(m, "Element")
     .def(py::init<const std::string &>())
     .def(py::init<int>())
@@ -80,10 +84,10 @@ void add_elem(py::module& m) {
     .def_property_readonly("is_hydrogen", &Element::is_hydrogen)
     .def_property_readonly("is_metal", &Element::is_metal)
     .def_property_readonly("it92", [](const Element& self) {
-        return IT92::get_ptr(self.elem);
+        return IT92::has(self.elem) ? &IT92::get(self.elem, 0) : nullptr;
     }, py::return_value_policy::reference_internal)
     .def_property_readonly("c4322", [](const Element& self) {
-        return C4322::get_ptr(self.elem);
+        return C4322::has(self.elem) ? &C4322::get(self.elem) : nullptr;
     }, py::return_value_policy::reference_internal)
     .def_property_readonly("neutron92", [](const Element& self) {
         return Neutron92::get(self.elem);  // a copy is created
@@ -93,6 +97,10 @@ void add_elem(py::module& m) {
         return "<gemmi.Element: " + std::string(self.name()) + ">";
     });
 
+  m.def("IT92_get_exact", [](gemmi::Element el, signed char charge) {
+      return IT92::get_exact(el.elem, charge);
+  }, py::return_value_policy::reference_internal);
+
   // resinfo.hpp
   py::enum_<ResidueInfo::Kind>(m, "ResidueInfoKind")
     .value("UNKNOWN", ResidueInfo::Kind::UNKNOWN)
