@@ -112,6 +112,7 @@ void add_mol(py::module& m) {
     .def_readwrite("entities", &Structure::entities)
     .def_readwrite("connections", &Structure::connections)
     .def_readwrite("cispeps", &Structure::cispeps)
+    .def_readwrite("mod_residues", &Structure::mod_residues)
     .def_readwrite("helices", &Structure::helices)
     .def_readwrite("sheets", &Structure::sheets)
     .def_readwrite("assemblies", &Structure::assemblies)
@@ -157,6 +158,9 @@ void add_mol(py::module& m) {
     .def("setup_cell_images", &Structure::setup_cell_images)
     .def("add_entity_types", (void (*)(Structure&, bool)) &add_entity_types,
          py::arg("overwrite")=false)
+    .def("add_conect", &Structure::add_conect,
+         py::arg("serial1"), py::arg("serial2"), py::arg("order"))
+    .def("clear_conect", [](Structure& self) { self.conect_map.clear(); })
     .def("assign_subchains", &assign_subchains,
          py::arg("force")=false, py::arg("fail_if_unknown")=true)
     .def("ensure_entities", &ensure_entities)
@@ -168,13 +172,17 @@ void add_mol(py::module& m) {
     .def("remove_waters", remove_waters<Structure>)
     .def("remove_ligands_and_waters", remove_ligands_and_waters<Structure>)
     .def("store_deuterium_as_fraction", &store_deuterium_as_fraction)
-    .def("assign_serial_numbers", (void (*)(Structure&)) &assign_serial_numbers)
+    .def("assign_serial_numbers", (void (*)(Structure&, bool)) &assign_serial_numbers,
+         py::arg("numbered_ter")=false)
     .def("shorten_chain_names", &shorten_chain_names)
-    .def("expand_ncs", &expand_ncs, py::arg("how"))
+    .def("expand_ncs", &expand_ncs, py::arg("how"), py::arg("merge_dist")=0.2)
     .def("transform_to_assembly",
-         [](Structure& st, const std::string& assembly_name, HowToNameCopiedChain how) {
-        return transform_to_assembly(st, assembly_name, how, nullptr);
-    }, py::arg("assembly_name"), py::arg("how"))
+         [](Structure& st, const std::string& assembly_name, HowToNameCopiedChain how,
+            bool keep_spacegroup, double merge_dist) {
+        return transform_to_assembly(st, assembly_name, how, nullptr,
+                                     keep_spacegroup, merge_dist);
+    }, py::arg("assembly_name"), py::arg("how"), py::arg("keep_spacegroup")=false,
+       py::arg("merge_dist")=0.2)
     .def("calculate_box", &calculate_box, py::arg("margin")=0.)
     .def("calculate_fractional_box", &calculate_fractional_box, py::arg("margin")=0.)
     .def("clone", [](const Structure& self) { return new Structure(self); })
@@ -521,8 +529,10 @@ void add_mol(py::module& m) {
                             HowToNameCopiedChain how) {
         return make_assembly(assembly, model, how, nullptr);
   });
+  m.def("expand_ncs_model", &expand_ncs_model);
   m.def("merge_atoms_in_expanded_model", &merge_atoms_in_expanded_model,
-        py::arg("model"), py::arg("cell"), py::arg("max_dist")=0.2);
+        py::arg("model"), py::arg("cell"), py::arg("max_dist")=0.2,
+        py::arg("compare_serial")=true);
 
   // select.hpp
   py::class_<FilterProxy<Selection, Model>> pySelectionModelsProxy(m, "SelectionModelsProxy");
