@@ -52,8 +52,7 @@ if [ -z "${NO_DOCTEST-}" ]; then
 fi
 
 flake8 docs/ examples/ tests/ tools/ setup.py
-# We want to avoid having '::' in pydoc from pybind11.
-$PYTHON -m pydoc gemmi | grep :: ||:
+pybind11-stubgen --dry-run --exit-code gemmi
 
 
 # Usually, we stop here. Below are more extensive checks below that are run
@@ -68,16 +67,20 @@ if [ $1 = i ]; then
 fi
 
 if [ $1 = m -o $1 = a ]; then
-    echo 'Creating, compiling and removing test_mmdb.cpp'
-    echo 'Example 1'
+    echo 'Creating, compiling and removing test_mmdb{1,2}.cpp'
+    echo 'Example 1: gemmi -> mmdb'
     cmd="c++ -O -Wall -Wextra -pedantic -Wshadow -Iinclude test_mmdb.cpp \
         -lmmdb2 -Lbuild -lgemmi_cpp -lz -o test_mmdb"
     awk '/Example 1/,/^}/' include/gemmi/mmdb.hpp > test_mmdb.cpp
-    $cmd
-    echo 'Example 2'
+    ${cmd}1
+    echo "Converting tests/1orc.pdb to /tmp/example1.pdb"
+    ./test_mmdb1 tests/1orc.pdb /tmp/example1.pdb
+    echo 'Example 2: mmdb -> gemmi'
     awk '/Example 2/,/^}/' include/gemmi/mmdb.hpp > test_mmdb.cpp
-    $cmd
-    rm -f test_mmdb.cpp
+    ${cmd}2
+    echo "Converting tests/1orc.pdb to /tmp/example2.pdb"
+    ./test_mmdb2 tests/1orc.pdb /tmp/example2.pdb
+    rm -f test_mmdb1.cpp test_mmdb2.cpp test_mmdb1 test_mmdb2
 fi
 
 if [ $1 = h -o $1 = a ]; then
